@@ -1,4 +1,5 @@
 import ast
+import html
 import os
 import sys
 from collections import defaultdict, Counter
@@ -18,7 +19,7 @@ from pure_eval import Evaluator, is_expression_interesting
 from stack_data.utils import (
     truncate, unique_in_order, line_range,
     frame_and_lineno, iter_stack, collapse_repeated, group_by_key_func,
-    cached_property, is_frame)
+    cached_property, is_frame, _pygmented_with_ranges)
 
 RangeInLine = NamedTuple('RangeInLine',
                          [('start', int),
@@ -160,16 +161,18 @@ class Options:
     it will be truncated with a gap in the middle. 
     """
     def __init__(
-            self,
+            self, *,
             before: int = 3,
             after: int = 1,
             include_signature: bool = False,
             max_lines_per_piece: int = 6,
+            pygments_formatter=None
     ):
         self.before = before
         self.after = after
         self.include_signature = include_signature
         self.max_lines_per_piece = max_lines_per_piece
+        self.pygments_formatter = pygments_formatter
 
     def __repr__(self):
         keys = sorted(self.__dict__)
@@ -318,7 +321,10 @@ class Line(object):
     def render(
             self,
             markers: Iterable[MarkerInLine] = (),
+            *,
             strip_leading_indent: bool = True,
+            pygmented: bool = False,
+            escape_html: bool = False
     ) -> str:
         """
         Produces a string for display consisting of .text
@@ -326,6 +332,15 @@ class Line(object):
         If strip_leading_indent is true (the default) then leading spaces
         common to all lines in this frame will be excluded.
         """
+        if pygmented:
+            assert not markers, "Cannot use pygmented with markers"
+            assert self.frame_info.options.pygments_formatter, "Must set a pygments formatter in Options"
+            start_line, lines = self.frame_info._pygmented_scope_lines
+            result = lines[self.lineno - start_line]
+            if strip_leading_indent:
+                result = result.replace(self.text[:self.leading_indent], "", 1)
+            return result
+
         text = self.text
 
         # This just makes the loop below simpler
@@ -341,7 +356,10 @@ class Line(object):
         original_start = start
 
         for marker in markers:
-            parts.append(text[start:marker.position])
+            text_part = text[start:marker.position]
+            if escape_html:
+                text_part = html.escape(text_part)
+            parts.append(text_part)
             parts.append(marker.string)
 
             # Ensure that start >= leading_indent
@@ -366,7 +384,7 @@ def markers_from_ranges(
             continue
 
         start_string, end_string = converted
-        if not isinstance(start_string, str) and isinstance(end_string, str):
+        if not (isinstance(start_string, str) and isinstance(end_string, str)):
             raise TypeError("converter should return None or a pair of strings")
 
         markers += [
@@ -376,6 +394,25 @@ def markers_from_ranges(
     return markers
 
 
+def style_with_executing_node(style, modifier):
+    from pygments.styles import get_style_by_name
+    if isinstance(style, str):
+        style = get_style_by_name(style)
+
+    class NewStyle(style):
+        for_executing_node = True
+
+        styles = {
+            **style.styles,
+            **{
+                k.ExecutingNode: v + " " + modifier
+                for k, v in style.styles.items()
+            }
+        }
+
+    return NewStyle
+
+
 class RepeatedFrames:
     """
     A sequence of consecutive stack frames which shouldn't be displayed because
@@ -473,6 +510,8 @@ class FrameInfo(object):
             cls,
             frame_or_tb: Union[FrameType, TracebackType],
             options: Optional[Options] = None,
+            *,
+            collapse_repeated_frames: bool = True
     ) -> Iterator[Union['FrameInfo', RepeatedFrames]]:
         """
         An iterator of FrameInfo and RepeatedFrames objects representing
@@ -482,10 +521,6 @@ class FrameInfo(object):
         Pass either a frame object or a traceback object,
         and optionally an Options object to configure.
         """
-        def _frame_key(x):
-            frame, lineno = frame_and_lineno(x)
-            return frame.f_code, lineno
-
         stack = list(iter_stack(frame_or_tb))
 
         # Reverse the stack from a frame so that it's in the same order
@@ -494,9 +529,20 @@ class FrameInfo(object):
         if is_frame(frame_or_tb):
             stack = stack[::-1]
 
+        def mapper(f):
+            return cls(f, options)
+
+        if not collapse_repeated_frames:
+            yield from map(mapper, stack)
+            return
+
+        def _frame_key(x):
+            frame, lineno = frame_and_lineno(x)
+            return frame.f_code, lineno
+
         yield from collapse_repeated(
             stack,
-            mapper=lambda f: cls(f, options),
+            mapper=mapper,
             collapser=RepeatedFrames,
             key=_frame_key,
         )
@@ -668,6 +714,36 @@ class FrameInfo(object):
             if isinstance(stmt, (ast.FunctionDef, ast.ClassDef, ast.Module)):
                 return stmt
 
+    @cached_property
+    def _pygmented_scope_lines(self) -> Optional[Tuple[int, List[str]]]:
+        # noinspection PyUnresolvedReferences
+        from pygments.formatters import HtmlFormatter
+
+        formatter = self.options.pygments_formatter
+        scope = self.scope
+        assert scope and formatter
+
+        if isinstance(formatter, HtmlFormatter):
+            formatter.nowrap = True
+
+        atok = self.source.asttokens()
+        node = self.executing.node
+        if node and getattr(formatter.style, "for_executing_node", False):
+            scope_start = atok.get_text_range(scope)[0]
+            start, end = atok.get_text_range(node)
+            start -= scope_start
+            end -= scope_start
+            ranges = [(start, end)]
+        else:
+            ranges = []
+
+        code = atok.get_text(scope)
+        lines = _pygmented_with_ranges(formatter, code, ranges)
+
+        start_line = line_range(scope)[0]
+
+        return start_line, lines
+
     @cached_property
     def variables(self) -> List[Variable]:
         """
@@ -684,7 +760,7 @@ class FrameInfo(object):
             pair
             for pair in evaluator.find_expressions(scope)
             if is_expression_interesting(*pair)
-        ]
+        ]  # type: List[Tuple[ast.AST, Any]]
 
         if isinstance(scope, ast.FunctionDef):
             for node in ast.walk(scope.args):
